/*
 * Selector.java February 2007
 *
 * Copyright (C) 2007, Niall Gallagher <niallg@users.sf.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA  02111-1307  USA
 */

package org.simpleframework.http.core;

/**
 * The <code>Selector</code> interface represents an object which is
 * used to process collection events. The sequence of events that
 * typically take place is for the collection to start, if not all
 * of the bytes can be consumed it selects, and finally when all of
 * the bytes within the entity have been consumed it is ready.
 * <p>
 * The start event is used to immediately consume bytes form the
 * underlying transport, it does not require a select to determine
 * if the socket is read ready which provides an initial performance
 * enhancement. Also when a response has been delivered the next
 * request from the pipeline is consumed immediately.
 * <p>
 * The select event is used to register the connected socket with a
 * Java NIO selector which can efficiently determine when there are
 * bytes ready to read from the socket. Finally, the ready event
 * is used when a full HTTP entity has been collected from the 
 * underlying transport. On such an event the request and response
 * can be handled by a container.
 * 
 * @author Niall Gallagher
 * 
 * @see org.simpleframework.http.core.Collector
 */
interface Selector extends Initiator {
  
   /**
    * The start event is used to immediately consume bytes form the
    * underlying transport, it does not require a select to check
    * if the socket is read ready which improves performance. Also,
    * when a response has been delivered the next request from the 
    * pipeline is consumed immediately.     
    * 
    * @param collector this is the collector used to collect data
    */
   public void start(Collector collector) throws Exception;
   
   /**
    * The select event is used to register the connected socket with 
    * a Java NIO selector which can efficiently determine when there 
    * are bytes ready to read from the socket. 
    *      
    * @param collector this is the collector used to collect data
    */
   public void select(Collector collector) throws Exception;
   
   /**
    * The ready event is used when a full HTTP entity has been 
    * collected from the underlying transport. On such an event the 
    * request and response can be handled by a container.
    * 
    * @param collector this is the collector used to collect data
    */
   public void ready(Collector collector) throws Exception;
   
   /**
    * This method is used to stop the <code>Selector</code> so that
    * all resources are released. As well as freeing occupied memory
    * this will also stop all threads, which means that is can no
    * longer be used to collect data from the pipelines.    
    */
   public void stop() throws Exception;
}
