/*
 * SocketTransport.java February 2007
 *
 * Copyright (C) 2007, Niall Gallagher <niallg@users.sf.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA  02111-1307  USA
 */

package org.simpleframework.transport;

import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;
import java.util.Map;

import javax.net.ssl.SSLEngine;

import org.simpleframework.transport.reactor.Reactor;

/**
 * The <code>SocketTransport</code> object offers a transport that can
 * send and receive bytes in a non-blocking manner. The contract of
 * the <code>Transport</code> is that it must either write the data
 * it is asked to write or it must queue that data for delivery. For
 * the vast majority of cases data is written directly to the socket
 * without any need for queuing or selection for write ready events.
 * <p>
 * In the event that the client TCP window is full and writing would
 * block this makes use of a queue of buffers which can be used to 
 * append data to. The buffers are lazily instantiated so the memory
 * required is created only in the rare case that they are needed.
 * Once a buffer is full it is queued to an asynchronous thread where
 * the buffer queue is drained and sent to the client when the TCP
 * window of the client is capable of accepting it.
 * <p>
 * In order to improve the network performance of this transport the
 * default packet size sent to the TCP stack is four kilobytes. This
 * ensures that the fragments of response delivered to the TCP layer
 * are sufficiently large for optimum network performance.
 *
 * @author Niall Gallagher
 */
class SocketTransport implements Transport {
   
   /**
    * This creates packets with increasing sequence numbers.
    */
   private PacketAllocator allocator;

   /**
    * This is the underlying byte channel used to send the  data.
    */         
   private SocketChannel channel;    
   
   /**
    * This is the writer that is used to flush the buffer queue.
    */
   private Controller writer;
   
   /**
    * This is the socket that this transport is representing.
    */
   private Socket socket;

   /**
    * This is the current internal buffer used by this transport.
    */
   private Packet buffer;
  
   /**
    * This is used to determine if the transport has been closed.
    */
   private boolean closed;
   
   /**
    * Constructor for the <code>SocketTransport</code> object. This 
    * requires a reactor to perform asynchronous writes and also the
    * pipeline which is used to read and write data. This transport
    * will use a queue of buffers which are lazily initialized so as
    * to only allocate the memory on demand.
    *
    * @param socket this is used to read and write the data
    * @param reactor this is used to perform asynchronous writes
    */
   public SocketTransport(Socket socket, Reactor reactor) throws Exception {
      this(socket, reactor, 4096);
   }

   /**
    * Constructor for the <code>SocketTransport</code> object. This 
    * requires a reactor to perform asynchronous writes and also the
    * pipeline which is used to read and write data. This transport
    * will use a queue of buffers which are lazily initialized so as
    * to only allocate the memory on demand.
    *
    * @param socket this is used to read and write the data
    * @param reactor this is used to perform asynchronous writes
    * @param size this is the size of the buffers to be allocated
    */
   public SocketTransport(Socket socket, Reactor reactor, int size) throws Exception {
      this(socket, reactor, size, 3);
   }

   /**
    * Constructor for the <code>SocketTransport</code> object. This 
    * requires a reactor to perform asynchronous writes and also the
    * pipeline which is used to read and write data. This transport
    * will use a queue of buffers which are lazily initialized so as
    * to only allocate the memory on demand.
    *
    * @param socket this is used to read and write the data
    * @param reactor this is used to perform asynchronous writes
    * @param size this is the size of the buffers to be allocated    
    * @param allow this is the number of buffers that are allowed
    */
   public SocketTransport(Socket socket, Reactor reactor, int size, int allow) throws Exception {
     this.writer = new SocketController(socket, reactor);
     this.allocator = new PacketAllocator(size, allow);
     this.channel = socket.getChannel();
     this.socket = socket;
   }   
   
   /**
    * This method is used to get the <code>Map</code> of attributes 
    * by this pipeline. The attributes map is used to maintain details
    * about the connection. Information such as security credentials
    * to client details can be placed within the attribute map.
    *
    * @return this returns the map of attributes for this pipeline
    */   
   public Map getAttributes() {     
      return socket.getAttributes();
   }

   /**
    * This is used to acquire the SSL engine used for https. If the
    * pipeline is connected to an SSL transport this returns an SSL
    * engine which can be used to establish the secure connection
    * and send and receive content over that connection. If this is
    * null then the pipeline represents a normal transport. 
    *  
    * @return the SSL engine used to establish a secure transport
    */   
   public SSLEngine getEngine() {     
      return socket.getEngine();
   }

   /**
    * This method is used to acquire the <code>SocketChannel</code>
    * for the connection. This allows the server to acquire the input
    * and output streams with which to communicate. It can also be 
    * used to configure the connection and perform various network 
    * operations that could otherwise not be performed.
    *
    * @return this returns the socket used by this HTTP pipeline
    */      
   public SocketChannel getChannel() {      
      return socket.getChannel();
   }   
   
   /**
    * This is used to perform a non-blocking read on the transport.
    * If there are no bytes available on the input buffers then
    * this method will return zero and the buffer will remain the
    * same. If there is data and the buffer can be filled then this
    * will return the number of bytes read. Finally if the socket
    * is closed this will return a -1 value.
    *
    * @param data this is the buffer to append the bytes to
    *
    * @return this returns the number of bytes that were read 
    */ 
   public int read(ByteBuffer data) throws Exception {
      if(closed) {
         throw new TransportException("Transport is closed");         
      }      
      return channel.read(data);
   }
   
   /**
    * This method is used to deliver the provided buffer of bytes to
    * the underlying transport. Depending on the connection type the
    * array may be encoded for SSL transport or send directly. This
    * will buffer the bytes within the internal buffer to ensure 
    * that the response fragments are sufficiently large for the
    * network. Smaller packets result poorer performance.
    *
    * @param data this is the array of bytes to send to the client
    */   
   public  void write(ByteBuffer data) throws Exception{  
      Packet buffer = allocate();    
      
      if(closed) {
         throw new TransportException("Transport is closed");
      }     
      write(data, buffer);      
   }

   /**
    * This method is used to deliver the provided buffer of bytes to
    * the underlying transport. Depending on the connection type the
    * array may be encoded for SSL transport or send directly. This
    * will buffer the bytes within the internal buffer to ensure 
    * that the response fragments are sufficiently large for the
    * network. Smaller packets result in poorer performance.
    *
    * @param data this is the array of bytes to send to the client
    * @param buffer this is the buffer to append the data to
    */    
   private void write(ByteBuffer data, Packet buffer) throws Exception {     
     int ready = data.remaining();
     int space = buffer.space();
     
     if(ready <= space) { 
        buffer.append(data);
     } else { 
       int capacity = buffer.capacity(); 
       
       if(space < capacity) { 
         if(space > 0) { 
            buffer.append(data);
         } 
         send(buffer, false);        
         write(data);
       } else {
         send(buffer, data);
       } 
     } 
   }    
   
   /**
    * This is used to perform an asynchronous write of the provided
    * data. In order to achieve this the provided buffer is queued
    * to another thread which executes when the socket is ready to
    * write. If a blocking asynchronous write is requested this 
    * will wait for the write to complete before returning.
    *
    * @param data this is the buffer to be written asynchronously
    * @param block if this is true then this method will wait
    */
   private void send(Packet packet, boolean block) throws Exception {
      if(packet == buffer) {
         buffer = null;
      }
      writer.write(packet, block);        
   }
   
   /**
    * This is used to perform an asynchronous write of the provided
    * data. In order to achieve this the provided buffer is queued
    * to another thread which executes when the socket is ready to
    * write. If a blocking asynchronous write is requested this 
    * will wait for the write to complete before returning.
    *
    * @param this is a packet that is to be written asynchronously
    * @param data this is the buffer to be written synchronously 
    */
   private void send(Packet packet, ByteBuffer data) throws Exception {
      Packet wrap = allocator.allocate(data);  
      
      send(packet, false);
      send(wrap, true);
   }

   /**
    * This is used to flush the internal buffer to the underlying
    * socket. Flushing with this method is always non-blocking, so
    * if the socket is not write ready and the buffer can be queued
    * it will be queued and the calling thread will return.
    */
   public void flush() throws Exception {      
      if(closed) {
         throw new TransportException("Transport is closed");
      }
      flush(false);
   }

   /**
    * This is used to flush the internal buffer to the underlying
    * socket. Flushing with this method is can be blocking or not.
    * When blocking is required this will wait for the flush to 
    * finish regardless of whether the socket is write ready.
    *
    * @param block determines whether the flush method should block
    */
   public void flush(boolean block) throws Exception {
      if(buffer != null) {
         send(buffer, block);
      }
   }  
   
   /**
    * This is used to allocate the buffers used by this transport. 
    * Allocation of buffers can be done from a pool such that this
    * does not necessarily need to create a new buffer when called.
    * Also, if there is space in the existing buffer this can be
    * used and data is appended to the end of the buffer.
    *
    * @return this returns a buffer to append the data to
    */ 
   private Packet allocate() throws Exception {
      if(buffer == null) {
         buffer = allocator.allocate();
      }
      return buffer;
   }

   /**
    * This method is used to flush the internal buffer and close
    * the underlying socket. This method will not complete until
    * all buffered data is written and the underlying socket is
    * closed at which point this can be disposed of.
    */
   public void close() throws Exception {
      if(!closed) {              
         flush(true);
         writer.close();
         closed = true;
      }
   }
}
