/*
 * ExecutorReactor.java February 2007
 *
 * Copyright (C) 2007, Niall Gallagher <niallg@users.sf.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General 
 * Public License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA  02111-1307  USA
 */

package org.simpleframework.transport.reactor;

import java.util.concurrent.Executor;

/**
 * The <code>ExecutorReactor</code> is used to schedule operation for
 * execution using an <code>Executor</code> implementation. This can be
 * useful when the operations performed are time intensive. For example
 * if the operations performed a read of the underlying channel and 
 * then had to parse the contents of the payload. Such operations would
 * reduce the performance of the reactor if it could not delegate to
 * some other form of executor, as it would delay their execution.
 *
 * @author Niall Gallagher
 */
public class ExecutorReactor implements Reactor {

  /**
   * This is used to distribute the ready operations for execution.
   */         
  private final Distributor exchange;

  /**
   * This is used to execute the operations that ready to run.
   */ 
  private final Executor executor;
  
  /**
   * Constructor for the <code>ExecutorReactor</code> object. This is
   * used to create a reactor that can delegate to the executor. This
   * also accepts the operations it is interested in, the value is
   * taken from the <code>SelectionKey</code> object. A bit mask can
   * be used to show interest in several operations at once.
   *
   * @param executor this is the executor used to run the operations
   */  
  public ExecutorReactor(Executor executor) throws Exception {
     this(executor, 1);
  }
  
  /**
   * Constructor for the <code>ExecutorReactor</code> object. This is
   * used to create a reactor that can delegate to the executor. This
   * also accepts the operations it is interested in, the value is
   * taken from the <code>SelectionKey</code> object. A bit mask can
   * be used to show interest in several operations at once.
   *
   * @param executor this is the executor used to run the operations
   * @param count this is the number of distributors to be used
   */  
  public ExecutorReactor(Executor executor, int count) throws Exception {    
     this(executor, count, 300000);
  }  

  /**
   * Constructor for the <code>ExecutorReactor</code> object. This is
   * used to create a reactor that can delegate to the executor. This
   * also accepts the operations it is interested in, the value is
   * taken from the <code>SelectionKey</code> object. A bit mask can
   * be used to show interest in several operations at once.
   *
   * @param executor this is the executor used to run the operations
   * @param count this is the number of distributors to be used
   * @param expiry the length of time to maintain and idle operation
   */    
  public ExecutorReactor(Executor executor, int count, long expiry) throws Exception {    
    this.exchange = new PartitionDistributor(executor, count, expiry);    
    this.executor = executor;
  }

  /**
   * This method is used to execute the provided operation without
   * the need to specifically check for I/O events. This is used if
   * the operation knows that the <code>SelectableChannel</code> is
   * ready, or if the I/O operation can be performed without knowing
   * if the channel is ready. Typically this is an efficient means
   * to perform a poll rather than a select on the channel.
   *
   * @param task this is the task to execute immediately
   */   
  public void process(Operation task) throws Exception {
     executor.execute(task);          
  }
  
  /**        
   * This method is used to execute the provided operation when there
   * is an I/O event that task is interested in. This will used the
   * operations <code>SelectableChannel</code> object to determine 
   * the events that are ready on the channel. If this reactor is
   * interested in any of the ready events then the task is executed.
   *
   * @param task this is the task to execute on interested events
   * @param require this is the bit-mask value for interested events
   */  
  public void process(Operation task, int require) throws Exception {         
     exchange.process(task, require);    
  }        

  /**
   * This is used to stop the reactor so that further requests to
   * execute operations does nothing. This will clean up all of 
   * the reactors resources and unregister any operations that are
   * currently awaiting execution. This should be used to ensure
   * any threads used by the reactor gracefully stop.
   */   
  public void stop() throws Exception {
     exchange.close();
  }
}




